<?php
/*
Plugin Name: BeBuilder & Betheme Options Migrate Tool
Plugin URI: https://muffingroup.com
Description: BeBuilder & Betheme Options Migrate Tool. Works with Betheme 27.0+
Author: Muffin Group
Author URI: https://muffingroup.com
Version: 4.1

Changelog:

4.1.
- nested wraps supported
- any structure of the mfn-page-items array supported
- replace local CSS and regenerate CSS file

*/

function mfn_migrate_menu() {
	add_submenu_page(
		'tools.php',
		'BeBuilder & Betheme Options Migrate Tool 4.1',
		'Betheme Migrate Tool',
		'edit_theme_options',
		'mfn_migrate_cb',
		'mfn_migrate_cb'
	);
}
add_action('admin_menu', 'mfn_migrate_menu');

/**
 * Replace & regenerate CSS
 */

function mfn_migrate_css( $old_url, $new_url ){

	global $wpdb;

	// local CSS styles

	$wpdb->query($wpdb->prepare( "UPDATE $wpdb->postmeta
		SET `meta_value` =
		REPLACE( meta_value, %s, %s)
		WHERE `meta_key` = 'mfn-page-local-style'
	", $old_url, $new_url ));

	// CSS files

	$items = get_posts( array(
		'post_type' => array( 'page', 'post', 'template', 'portfolio', 'product' ),
		'post_status' => 'publish',
		'posts_per_page' => -1,
	) );

	if( ! empty( $items ) && is_array( $items ) ){
		foreach( $items as $item ){
			if( get_post_meta( $item->ID, 'mfn-page-local-style') ){
				$mfn_styles = json_decode( get_post_meta( $item->ID, 'mfn-page-local-style', true ), true );
				Mfn_Helper::generate_css( $mfn_styles, $item->ID );
			}
		}
	}

	return true;

}

/**
 * Theme Options
 */

function mfn_migrate_theme_options( $old_url, $new_url ){

	$count = 0;

	$data = get_option( 'betheme' );

	if( $data && is_array( $data ) ){
		foreach( $data as $key => $option ){
			if( is_string($option) && strpos($option, $old_url) !== false ){
				$data[$key] = str_replace( $old_url, $new_url, $option );
				$count ++;
			}
		}
	}

	update_option( 'betheme', $data );

	echo '<p>Theme Options changes: <strong>'. $count .'</strong></p>';

}

/**
 * Replace URLs recursively
 */

function mfn_migrate_replace_urls_recursively( $data, $old_url, $new_url ) {

	global $mfn_migrate_count;

	// regular expression to match #integer (example: a.png#23)
	$pattern = '/#\d+/';

	foreach ($data as $key => $value) {
		if (is_array($value)) {
			// recursively handle nested arrays
			$data[$key] = mfn_migrate_replace_urls_recursively($value, $old_url, $new_url);
		} elseif ( is_string($value) && strpos($value, $old_url) !== false ) {
			// replace old URL with new URL
			$value = preg_replace($pattern, '', $value);
			$data[$key] = str_replace($old_url, $new_url, $value);
			$mfn_migrate_count++;
		}
	}

	return $data;

}

/**
 * BeBuilder
 */

function mfn_migrate_builder( $old_url, $new_url ){

	global $wpdb;
	global $mfn_migrate_count;

	$mfn_migrate_count = 0;

	$results = $wpdb->get_results( "SELECT * FROM $wpdb->postmeta
		WHERE `meta_key` = 'mfn-page-items'
	" );

	if( is_array( $results ) ){

		// posts loop -----------------

		foreach( $results as $result_key=>$result ){

			$meta_id = $result->meta_id;
			$post_id = $result->post_id;

			$meta_value = @unserialize( $result->meta_value );

			// Builder 2.0 Compatibility

			if( $meta_value === false ){
				$meta_value = unserialize( base64_decode( $result->meta_value ) );
			}

			// replace ----------------

			if( is_array( $meta_value ) ){

				// var_dump($post_id);
				// var_dump($meta_value);

				$meta_value = mfn_migrate_replace_urls_recursively( $meta_value, $old_url, $new_url );

				// var_dump($meta_value);
				// break;

				$meta_value = base64_encode( serialize( $meta_value ) );

				update_post_meta( $post_id, 'mfn-page-items', $meta_value );
				delete_post_meta( $post_id, 'mfn-page-object' );

			}

		}
	}

	echo '<p>BeBuilder changes: <strong>'. $mfn_migrate_count .'</strong></p>';

}

/**
 * BeBuilder & Betheme Options Migrate Tool
 */

function mfn_migrate_cb(){

	$safety_limit = 10;

	if( key_exists( 'mfn_migrate_nonce',$_POST ) ) {
		if ( wp_verify_nonce( $_POST['mfn_migrate_nonce'], basename(__FILE__) ) ) {

			$old_url = stripslashes(htmlspecialchars($_POST['old']));
			$new_url = stripslashes(htmlspecialchars($_POST['new']));

			if( strlen($old_url) < $safety_limit || strlen($new_url) < $safety_limit ){

				echo '<p><strong>For your own safety please use URLs longer than '. $safety_limit .' characters !</strong></p>';

			} elseif( strpos( $old_url, 'http' ) !== 0 || strpos( $new_url, 'http') !== 0 ){

				echo '<p><strong>URLs must begin with http:// or https:// !</strong></p>';

			} else {

				mfn_migrate_theme_options( $old_url, $new_url );

				mfn_migrate_builder( $old_url, $new_url );

				mfn_migrate_css( $old_url, $new_url );

				echo '<p><strong>All done. Have fun!</strong></p>';

			}
		} else {
			echo '<p><strong>Invalid Nonce !</strong></p>';
		}
	}

	?>
		<div class="wrap">

			<div id="icon-tools" class="icon32"></div>
			<h2><?php echo esc_html( get_admin_page_title() ); ?></h2>
			<br />

			<form action="" method="post">

				<input type="hidden" name="mfn_migrate_nonce" value="<?php echo wp_create_nonce(basename(__FILE__)); ?>" />

				<label style="width:50px; display:inline-block;">Find</label>
				<input type="text" name="old" value="" placeholder="Old URL" style="width:500px;" />
				<br />
				<br />

				<label style="width:50px; display:inline-block;">Replace</label>
				<input type="text" name="new" value="<?php echo home_url(); ?>/" style="width:500px;" />

				<input type="submit" name="submit" class="button button-primary" value="Replace" />

      </form>

		</div>
	<?php
}

?>
